/*
 * IBM Confidential
 * OCO Source Materials
 * ------------------
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta;

import javax.resource.ResourceException;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.CommException;
import javax.resource.spi.ResourceAdapterInternalException;

import com.ibm.j2ca.base.WBIActivationSpecWithXid;
import com.ibm.j2ca.base.WBIPollableResourceAdapterWithXid;
import com.ibm.j2ca.base.WBIResourceAdapter;
import com.ibm.j2ca.base.WBIResourceAdapterMetadata;
import com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid;
import com.ibm.j2ca.vta.inbound.VTAActivationSpecWithXid;
import com.ibm.j2ca.vta.inbound.VTAEventStoreWithXid;

/**
 * VTAResourceAdapter. This class represents a JCA ResourceAdapter. 
 * A resource adapter instance contains operations for lifecycle management and message endpoint setup.
 * A concrete implementation of this interface is required to be a JavaBean. 
 * This is a global property container for the VisualTestAdapter.
 */
public class VTAResourceAdapter extends WBIResourceAdapter 
		    implements  javax.resource.spi.ResourceAdapter, WBIPollableResourceAdapterWithXid
{

	//added by VTADev Team
	//private WBIResourceAdapterMetadata raMetaData = null;
	//generated by WAT
	private VTAWSAResourceAdapterMetadata raMetaData = null;

    /**
     * start
     * 
     * @param 	bootstrapCtx
     * @throws 	ResourceAdapterInternalException
     * @see 	javax.resource.spi.ResourceAdapter#start(javax.resource.spi.BootstrapContext)
     */
    public void start(BootstrapContext bootstrapCtx) throws ResourceAdapterInternalException {
        super.start(bootstrapCtx);
        getLogUtils().traceMethodExit(VTAResourceAdapter.class.getName(), "start()");
    }

    /**
     * It creates an EventStore when the Resource Adapter is deployed in the App server
     * 
     * createEventStore
     * 
     * @param 	activationSpec	inbound properties
     * @return 					an instance of VTAEventStore, ready for polling
     * @throws 					ResourceException
     * @see 					com.ibm.j2ca.base.WBIPollableResourceAdapterWithXid#createEventStore(com.ibm.j2ca.base.WBIActivationSpecWithXid;)
     */
     public EventStoreWithXid createEventStore(WBIActivationSpecWithXid activationSpec) throws ResourceException {
        getLogUtils().traceMethodEntrance(VTAResourceAdapter.class.getName(), "createEventStore()");
    	EventStoreWithXid eventStore = null;
        try{
        	eventStore = new VTAEventStoreWithXid((VTAActivationSpecWithXid)activationSpec, this);
        }
        catch(Exception e)
        {
        	throw new CommException(e.getMessage(), e);
        }
        
        getLogUtils().traceMethodExit(VTAResourceAdapter.class.getName(), "createEventStore()");
        return eventStore;
     }

    /**
     * @return an instance of VTAWSAResourceAdapterMetadata
     */
    public WBIResourceAdapterMetadata getResourceAdapterMetadata() throws ResourceException {
        boolean isTxnSupported = false;  // Is transaction supported? -> This is not used by VTADev Team 
        //added by VTADev Team
        if (raMetaData == null) {
        	raMetaData = new VTAWSAResourceAdapterMetadata("IBM WebSphere Adapter for Visual Test", "IBM", "6.2",isTxnSupported);
        }
        return raMetaData;    

       //This is generated by WAT 
       //return new VTAWSAResourceAdapterMetadata(VTAConstants.CONNECTOR_NAME, VTAConstants.CONNECTOR_VENDOR,VTAConstants.CONNECTOR_VERSION, isTxnSupported);
    }
    
    public String getAdapterID() {
    	return VTAConstants.VTA_ADAPTERID;
    }
}