/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta;

import javax.resource.cci.Record;

import com.ibm.despi.OutputAccessor;
import com.ibm.despi.OutputCursor;
import com.ibm.despi.exception.DESPIException;
import com.ibm.eismocker.rmidriver.DataMap;
import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.base.WBIStructuredRecord;
import com.ibm.j2ca.extension.metadata.Type;
import com.ibm.j2ca.vta.util.VTAUtil;
import commonj.connector.runtime.DataBindingException;
import commonj.connector.runtime.RecordHolderDataBinding;
import commonj.sdo.DataObject;

public class VTAStructuredRecord extends WBIStructuredRecord implements RecordHolderDataBinding{
	private static final long serialVersionUID = 5150851386271623504L; //value added by VATDev Team
	//added by VTADev Team
	private String bgVerb="";
	private boolean isBG=false;

	public VTAStructuredRecord(){
		super();
	}

    /**
     * The adapter should  populate the field represented by the xpath expression passed in.  
     *
     * @param  xpath  an absolute URL giving the base location of the image
     */
	public void extract(String xpath) throws DESPIException {
		getLogUtils().traceMethodEntrance(VTAStructuredRecord.class.getName(), "extract()");
		// TODO
		getLogUtils().traceMethodExit(VTAStructuredRecord.class.getName(), "extract()");
	}

    /**
     * The subclass should use the cursor/accessor pattern to populate the output data.   If copyValues is true, the adapter 
     * should populate the entire output.   If copyValues is false, the adapter should not populate the entire output, but 
     * rather just set up any data structures that the adapter might need before processing extract calls, 
     *
     * @param  copyValue  an absolute URL giving the base location of the image
     * @return  bValue    the image at the specified URL
     */
	public boolean getNext(boolean copyValues) throws DESPIException {
		/*getLogUtils().traceMethodEntrance(VTAStructuredRecord.class.getName(), "getNext()");
		boolean bValue = false;
		// TODO implement this
		getLogUtils().traceMethodExit(VTAStructuredRecord.class.getName(), "getNext()");
		return bValue;
		*/
		//For inbound processing.
		DataMap dataMap = (DataMap) this.getEISRepresentation();
		if(dataMap.getName().endsWith("BG")){// it is a BG, then get the root BO.
			String bgName = dataMap.getName();
			String bgVerb = (String)dataMap.get(VTAConstants.VERB);
			this.setBGVerb(bgVerb);
			String boName = bgName.substring(0, bgName.length()-2);
			dataMap = dataMap.getDataMap(boName);
		}
		Type metadata = this.getMetadata();
		OutputCursor topCursor = (OutputCursor)this.getTopLevelCursor();
		
		if(this.getOperationName().equalsIgnoreCase(WBIInteractionSpec.EXISTS_OP)){
			//Boolean value = new Boolean(true);
			Boolean value = (Boolean)this.getEISRepresentation();
			topCursor.startObject();
			OutputAccessor accessor = (OutputAccessor)topCursor.getAccessors().get(0);
			accessor.setBoolean(value.booleanValue());
			topCursor.completeObject();
			return true;			
		}
		if(this.getOperationName().equalsIgnoreCase(WBIInteractionSpec.RETRIEVE_ALL_OP)){
			return false;
		}
		
		topCursor.startObject();
		VTAUtil.copyDataMapToRecord(dataMap, topCursor, metadata,this.getLogUtils());
		topCursor.completeObject();
		
		return true;
	}
	
    /**
     * clone
     * @return record     the image at the specified URL
     */
	public Object clone()  {
	    getLogUtils().traceMethodEntrance(VTAStructuredRecord.class.getName(), "clone()");
		VTAStructuredRecord record = new VTAStructuredRecord();
		// TODO implement method
		getLogUtils().traceMethodExit(VTAStructuredRecord.class.getName(), "clone()");
		return record;
	}

	 /**
      * close
      * 
      */
	public void close() {
	    getLogUtils().traceMethodEntrance(VTAStructuredRecord.class.getName(), "close()");
		// TODO Auto-generated method stub	
		getLogUtils().traceMethodExit(VTAStructuredRecord.class.getName(), "close()");
	}
	
	public Record getRecord() throws DataBindingException {
		// TODO Auto-generated method stub
		return null;
	}

	public void setRecord(Record arg0) throws DataBindingException {
		// TODO Auto-generated method stub
		
	}

	public DataObject getDataObject() throws DataBindingException {
		// TODO Auto-generated method stub
		return null;
	}

	public void setDataObject(DataObject arg0) throws DataBindingException {
		// TODO Auto-generated method stub	
	}
	public void setBGVerb(String verb){
		this.bgVerb = verb;
	}
	public String getBGVerb(){
		return this.bgVerb;
	}
	public void setIsBG(boolean bg){
		this.isBG = bg;
	}
	public boolean getIsBG(){
		return this.isBG;
	}
}
