/*
 * IBM Confidential
 * OCO Source Materials
 * (C) Copyright IBM 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.emd.description;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.logging.Level;

import javax.xml.namespace.QName;

import com.ibm.j2ca.base.TopLevelVerbs;
import com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl;
import com.ibm.j2ca.extension.emd.description.WBIMetadata;
import com.ibm.j2ca.vta.emd.discovery.VTAAttributeType;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataImportConfiguration;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataObject;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.SchemaDefinition;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataObjectResponse;


/**
 * This class will fully describe the data for the base classes to generate an XSD.
 */
public class VTADataDescription extends WBIDataDescriptionImpl {

    private static final String CLASSNAME = "VTADataDescription"; 

    //added by VTADev Team
	private StringBuffer includeXSDs = new StringBuffer();
	private MetadataImportConfiguration[] selectedSet;
	private StringBuffer includeXSDsSecondary = new StringBuffer();

	//this constructor has been added by VTADev Team
	public VTADataDescription(VTAMetadataObject object, MetadataImportConfiguration[] selectedSet) throws MetadataException {
		super.setMetadataObject(object);
		super.setAliasTag(object.getAliasTag());
		this.selectedSet = selectedSet;
	}

	//this method has been added by VTADev Team
	public void populateSchemaDefinitions() throws MetadataException {
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodEntrance("VTADataDescription", "populateSchemaDefinitions");

		try {
			prepareChildSchemaFiles();
			VTAMetadataObject object = (VTAMetadataObject) getMetadataObject();
			// Extract the content after the tag for ComplexType> This is needed
			// to add the annotation for object level ASI
			String complexTypeInfo = object.getContent().substring(0, object.getContent().indexOf(">") + 1) + System.getProperty("line.separator");
			String content;
			String header = object.getHeader();
			int encoding = header.indexOf("encoding");
			if (encoding != -1) {
				String encodingName = header.substring(encoding + 10, header.indexOf("\"", encoding + 10));
				if (!encodingName.equalsIgnoreCase("UTF-8")) {
					header = header.substring(0, encoding + 10) + "UTF-8" + header.substring(header.indexOf("\"", encoding + 10));
				}
			}
			content = header + includeXSDs.toString() + getSerializedImports(this.getImportNameSpaces()) + System.getProperty("line.separator") + getSerializedAnnotationForEMD(getASISchemaName())
					+ complexTypeInfo + getSerializedMetadata(this.getMetadataForBusinessObject()) + object.getContent().substring(object.getContent().indexOf(">") + 1);
			// Add defn to schema defintion list
			URI uri = new URI(this.getRelativePath() + getBOName() + ".xsd");
			put(object.getTargetNameSpace(), uri, content);
		} catch (Exception e) {
			throw new MetadataException("Unable to create Data Description from Object selected", e);
		}
		
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodExit("VTADataDescription", "populateSchemaDefinitions");
	}

	
    /** 
     * This method returns the XML schema definition type that represents the
     * element or field of the object definition.
     *
     * @param 	attrName
     * @return				The correct type of the object
     * @see 				com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getType(java.lang.String)
     */
    public String getType(String attrName) {
        /* generated by WAT. not used by VTADev team
    	if (getCardinality(attrName) == Constants.N_CARDINALITY) {
            return attrName.toLowerCase() + ":" + StringCaseChanger.toCamelCase(attrName);
        }
        // TODO return the correct type.
        return Constants.XS_STRING;
        */
    	//added by VTADev team
		LinkedHashMap ht = this.getAttributeList();
		VTAAttributeType vtaAttrType = (VTAAttributeType) ht.get(attrName);
		return vtaAttrType.getType();
    }

    /**
     * For any children, prepare schema files for them.
     * @see com.ibm.j2ca.extension.emd.WBIDataDescriptionImpl#prepareChildSchemaFiles()
     */
    public void prepareChildSchemaFiles() throws MetadataException {
		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodEntrance("VTADataDescription", "prepareChildSchemaFiles");

		MetadataObjectResponse response = getMetadataObject().getChildren(null);
		VTAMetadataObject child = null;
		Iterator i = response.getObjectIterator();
		while (i.hasNext()) {
			child = (VTAMetadataObject) i.next();
			if (child.getBOType() != null) {
				if (child.getTargetNameSpace().equals(((VTAMetadataObject) this.getMetadataObject()).getTargetNameSpace()) && child.isPartOfSameXSD()) {
					includeXSDs.append("<" + child.getAliasTag() + "include schemaLocation=\"" + child.getBOName() + ".xsd\" />\n");
				}

				boolean found = false;
				for (int j = 0; j < selectedSet.length; j++) {
					VTAMetadataObject selectedObj = (VTAMetadataObject) ((VTAMetadataImportConfiguration) selectedSet[j]).getMetadataObject();
					if (selectedObj.getLocation().equals(child.getLocation())) {
						found = true;
						break;
					}
				}
				if (found)
					continue;
				// Generate the schemas for grand and following level children
				VTADataDescription desc = new VTADataDescription(child, selectedSet);
				desc.setRelativePath(this.getRelativePath());
				desc.setAliasTag(child.getAliasTag());
				desc.prepareChildSchemaFiles();
				SchemaDefinition[] schemas = desc.getSchemaDefinitions();
				for (int j = 0; j < schemas.length; j++) {
					SchemaDefinition defn = schemas[j];
					put(defn.getNamespace(), defn.getLocation(), defn.getContent());
				}
				// If not found continue to add the defn of the child object
				String complexTypeInfo = child.getContent().substring(0, child.getContent().indexOf(">") + 1) + System.getProperty("line.separator");
				String header = child.getHeader();
				int encoding = header.indexOf("encoding");
				if (encoding != -1) {
					String encodingName = header.substring(encoding + 10, header.indexOf("\"", encoding + 10));
					if (!encodingName.equalsIgnoreCase("UTF-8"))
						header = header.substring(0, encoding + 10) + "UTF-8" + header.substring(header.indexOf("\"", encoding + 10));
				}
				String content = header + desc.getIncludeXSDs() + getSerializedImports(this.getImportNameSpaces()) + System.getProperty("line.separator")
						+ getSerializedAnnotationForEMD(getASISchemaName()) + complexTypeInfo + getSerializedMetadata(this.getMetadataForBusinessObject())
						+ child.getContent().substring(child.getContent().indexOf(">") + 1);
				URI uri = null;
				try {
					uri = new URI(this.getRelativePath() + child.getBOName() + ".xsd");
				} catch (URISyntaxException use) {
					throw new MetadataException(use.getMessage(),use);
				}
				
				put(child.getTargetNameSpace(), uri, content);

			} else {

				VTADataDescription boDesc = new VTADataDescription(child, selectedSet);
				boDesc.setRelativePath(this.getRelativePath());
				boDesc.setAliasTag(child.getAliasTag());
				boDesc.populateSchemaDefinitions();
				SchemaDefinition[] schemas = boDesc.getSchemaDefinitions();
				for (int j = 0; j < schemas.length; j++) {
					SchemaDefinition defn = schemas[j];
					put(defn.getNamespace(), defn.getLocation(), defn.getContent());
				}
			}
		}

		if (super.getLogUtils().isTraceEnabled(Level.FINE))
			super.getLogUtils().traceMethodExit("VTADataDescription", "prepareChildSchemaFiles");
}

    /**
     *  Define the verbs supported by the top-level object.
     * 
     *	@return	list	List of supported verbs
     */
    public List getVerbs() {
        ArrayList list = new ArrayList();
        list.add(TopLevelVerbs.CREATE_TLV);
        list.add(TopLevelVerbs.UPDATE_TLV);
        list.add(TopLevelVerbs.DELETE_TLV);
        return list;
    }

    /**
     * This method returns an instance of the WBIMetadata object and represents
     * application specific information for the element or field in the object
     * definition. This corresponds to the annotation section for the element definition.
     *
     * @param 	attrname
     * @return 	attributeMetadata 	Returns WBIMetadata for a given Attribute
     */
    public WBIMetadata getMetadataForAttribute(String attrName) {
        /* generated WAT code not implemented by VTADev Team
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodEntrance(CLASSNAME, "getMetadataForAttribute");
        WBIMetadata attributeMetadata = new WBIMetadata();
        // TODO set the application specific information for the attribute in the business object here.
        WBIMetadataDiscoveryImpl.getLogUtils().traceMethodExit(CLASSNAME, "getMetadataForAttribute");
        return attributeMetadata;
        */
    	return null;
    }

    /**
     * This method returns an instance of the WBIMetadata object and represents
     * application specific information for the object definition. This corresponds
     * to the annotation section for the complexType definition.
     *
     * @return 	bometadata 	Returns WBIMetadata for a given BusinessObject
     */
    public WBIMetadata getMetadataForBusinessObject() {
        /*generated WAT code not implemented by VTADev Team
    	getLogUtils().traceMethodEntrance(CLASSNAME, "getMetadataForBusinessObject");
        WBIMetadata bometadata = new WBIMetadata();
        // TODO set the application specific information for the business object definition here.
        getLogUtils().traceMethodExit(CLASSNAME, "getMetadataForBusinessObject");
        return bometadata;
        */
    	return null;
    }

	/**
	 * This method is the default logic for the foundation classes to build a Container.
	 * It returns true when the complexType definition requires a Container 
	 * definition with the graph definition. The Container definition is used 
	 * when the adapter supports a RetrieveAll operation.
	 *
	 * @return	retValue	Returns true is the object is a container
	 */ 
    public boolean isContainer() {
        /* generated WAT code. not used by VTADev team code
    	getLogUtils().traceMethodEntrance(CLASSNAME, "isContainer");
        boolean retValue = false;
        String objName = getName().getLocalPart();
        if (objName.endsWith("Container") && !objName.equals(getBOName()))
            retValue = true;
        getLogUtils().traceMethodExit(CLASSNAME, "isContainer");
        return retValue;
        */
    	//added by VTADev team
    	boolean retValue = false;
		String objName = getName().getLocalPart();
		if (objName.endsWith("Container") && !objName.equals(getBOName()))
			retValue = true;
		return retValue;
    }

    /**
     * This method returns the element name that represents the field in the object.
     * This is the element name that would be used in the XML schema definition.
     *
     * @param 	attrName
     * @return 	formatted attrName
     * @see 	com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getAttributeName(java.lang.String)
     */
    public String getAttributeName(String attrName) {
        //return StringCaseChanger.toCamelCase(attrName); generated WAT code
    	return attrName; //added by VTADev Team
    }

    /**
     * This method returns a list of ImportedNameSpace instances that should be 
     * imported in the business object schema.
     *
     * @return 	list
     * @throws 	MetadataException
     * @see 	com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getImportNameSpaces()
     */
    public List getImportNameSpaces() throws MetadataException {
        ArrayList list = new ArrayList();
        /* WAT generated code. not implemented in VTADev team code
        ImportedNameSpace namespace = new ImportedNameSpace();
        namespace.setLocation(Constants.VTAASI_XSD);
        namespace.setNameSpace(Constants.ASI_TARGET_NAMESPACE);
        list.add(namespace);
        getLogUtils().traceMethodExit(CLASSNAME, "getImportNameSpaces");
        */
        return list;
    }

    /**
     * This method returns the NameSpaces listed in the XML schema definition. 
     * Typically these are application specific information schema definition 
     * namespaces. Note that this list is for outside namespaces only; child
     * object namespaces are included by the Adapter Foundation Classes.
     *
     * @return 	list
     * @see 	com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getNameSpaces()
     */
    public List getNameSpaces() {
        /* This is WAT generated not used by VTADev team
    	getLogUtils().traceMethodEntrance(CLASSNAME, "getNameSpaces");
        // TODO implement this method
        ArrayList list = new ArrayList();
        getLogUtils().traceMethodExit(CLASSNAME, "getNameSpaces");
        return list;
        */
		ArrayList list = new ArrayList();
		list.add(new QName("http://www.ibm.com/xmlns/prod/websphere/j2ca/vta", "pasi"));
		return list;
    }

	/**
	 * This method returns the target namespace that represents the application
	 * specific information schema.
	 *
	 * @return Constants.ASI_TARGET_NAMESPACE
	 */
    public String getASISchemaName() {
        //return Constants.ASI_TARGET_NAMESPACE; //generated WAT code
    	return null; //added by VTADev Team
    }

    /**
     * This method returns the cardinality for an elements in the business object
     * definition. This value is used to formulate the maxOccurs and minOccurs tag.
     *
     * @return	String representing the cardinality of the attribute
     * @see 	com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getCardinality(java.lang.String)
     */
    public String getCardinality(String attrName) {
        // TODO Implement this method to return the attribute cardinality
    	//added by VTADev team
		LinkedHashMap ht = this.getAttributeList();
		VTAAttributeType vtaAttrType = (VTAAttributeType) ht.get(attrName);
		return vtaAttrType.getCardinality();
    }

    /**
     * This method returns the maxLength for the elements in the business object
     * definition. This is used to fill in the maxLength tag for the XML schema definition.
     *
     * @return	int representing the max length of the attribute
     * @see 	com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getMaxLength(java.lang.String)
     */
    public int getMaxLength(String name) {
    	// TODO Implement this method to return the attribute maximum length
        return 0;
    }

    /**
     * This method returns true if the element is marked as required in the XML
     * schema definition. Otherwise, this method returns false.
     *
     * @return	boolean value to represent if the attribute is required
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getRequired(java.lang.String)
     */
    public boolean getRequired(String attrName) {
    	// TODO Implement this method to return true if the attribute is required
        return false;
    }

    /** 
     * This method returns the Iterator for the child objects of the DataDescription. 
     *
     * @return	Iterator of child objects
     * @see com.ibm.j2ca.extension.emd.description.WBIDataDescriptionImpl#getChildList()
     */
    public Iterator getChildList() throws MetadataException{
        return (this.getMetadataObject().getChildren(null)).getObjectIterator();
    }
    
    //the following methods have been added by VTADev team
    public boolean isTopLevel() {
		return false;
	}

	protected String getIncludeXSDs() {
		return this.includeXSDs.toString();
	}

	protected String getIncludeXSDsSecondary() { // cr84766
		return this.includeXSDsSecondary.toString();
	}
}