/*
 * YourCompany Confidential
 * OCO Source Materials
 * (C) Copyright YourCompany 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.emd.discovery;

import com.ibm.j2ca.extension.emd.discovery.WBIAdapterTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAInboundConnectionType;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAOutboundConnectionType;
import com.ibm.j2ca.vta.emd.Constants;
import commonj.connector.metadata.MetadataException;

/**
 * This class defines the AdapterType for VTA, along
 * with the inbound and outbound connections.
 */
public class VTAAdapterType extends WBIAdapterTypeImpl {
    private static final String ADAPTERTYPE_PROPERTY = "AdapterType";
    public static final String VENDOR = "IBM";  // TODO modify vendor name
    public static final String VERSION = "6.2";		// TODO modify version
   
    /**
     * Create a new instance of the VTAAdapterType
     * Define the outbound and inbound connections for VTA.
     *
     * @throws MetadataException
     */
    public VTAAdapterType()throws MetadataException{
    	/* 
    	 * The WBIAdapterTypeImpl constructor requires the following parameters:
    	 * 1. The name of the class representing the ResourceAdapter class. This is used to create property groups for ResourceAdapter in enterprise metadata discovery. 
    	 * 2. Number of outbound connections 
    	 * 3. Number of inbound connections
    	 */
        super(Constants.RESOURCE_ADAPTER_BEAN_NAME, 2, 1);
        setId(Constants.ADAPTER_NAME);
        setDisplayName(Constants.ADAPTER_NAME);
        setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(ADAPTERTYPE_PROPERTY));
        setVendor(VENDOR);
        setVersion(VERSION);
        setOutboundConnections();
        setInboundConnections();
    }

    /**
     * Define inbound connections
     * <p>
     * Set supportedInMetadataService to true for connections that you want
     * WebSphere Integration Developer to display when performing enterprise
     * metadata discovery. Add the connections to adapterType using the 
     * addInboundConnectionType() method.
     *
     * @throws MetadataException
     */
    private void setInboundConnections() throws MetadataException {
        VTAInboundConnectionType inConnTypeForRuntime = new VTAInboundConnectionType(this);
        addInboundConnectionType(inConnTypeForRuntime);
        inConnTypeForRuntime.setResourceAdapterJavaBean(Constants.RESOURCE_ADAPTER_BEAN_NAME);
        inConnTypeForRuntime.setActivationSpecJavaBean(Constants.ACTIVATION_SPEC);
    }

    /**
     * Define outbound connections
     * <p>
     * Set supportedInMetadataService to true for connections that you want
     * WebSphere Integration Developer to display when performing enterprise
     * metadata discovery. Add the connections to adapterType using the 
     * addOutboundConnectionType() method.
     *
     * @throws MetadataException
     */
    private void setOutboundConnections() throws MetadataException {
        VTAOutboundConnectionType outConnTypeForRuntime = new VTAOutboundConnectionType(this);
        VTAOutboundConnectionType outConnTypeForMetadata = new VTAOutboundConnectionType(this);
        outConnTypeForMetadata.setManagedConnectionFactoryJavaBean(Constants.MANAGED_CONNECTION_FACTORY_NAME);
        outConnTypeForRuntime.setManagedConnectionFactoryJavaBean(Constants.MANAGED_CONNECTION_FACTORY_NAME);
        outConnTypeForMetadata.setResourceAdapterJavaBean(Constants.RESOURCE_ADAPTER_BEAN_NAME);
        outConnTypeForRuntime.setResourceAdapterJavaBean(Constants.RESOURCE_ADAPTER_BEAN_NAME);
        outConnTypeForMetadata.setIsSupportedInMetadataService(true);
        outConnTypeForMetadata.setIsSupportedAtRuntime(false);
        outConnTypeForRuntime.setIsSupportedInMetadataService(false);
        outConnTypeForRuntime.setIsSupportedAtRuntime(true);
        addOutboundConnectionType(outConnTypeForMetadata);
        addOutboundConnectionType(outConnTypeForRuntime);
    }
    
}
