/*
 * IBM Confidential
 * OCO Source Materials
 * (C) Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.emd.discovery;

import java.util.logging.Level;
import javax.resource.spi.ResourceAdapterInternalException;
import com.ibm.j2ca.extension.emd.discovery.*;
import com.ibm.j2ca.extension.emd.discovery.connection.*;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.base.WBIManagedConnectionFactory;

import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.description.ServiceDescription;
import commonj.connector.metadata.discovery.*;
import commonj.connector.metadata.discovery.connection.MetadataConnection;
import commonj.connector.tool.ToolContext;
import commonj.connector.metadata.discovery.properties.PropertyGroup;
import commonj.connector.metadata.discovery.properties.SingleValuedProperty;

import com.ibm.j2ca.vta.VTAResourceAdapter;
import com.ibm.j2ca.vta.emd.Constants;
import com.ibm.j2ca.vta.emd.description.VTAInboundServiceDescription;
import com.ibm.j2ca.vta.emd.description.VTAOutboundServiceDescription;

//added by VTADev team
import java.util.logging.LogRecord;
import com.ibm.j2ca.extension.emd.discovery.WBIAdapterTypeImpl;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAOutboundConnectionConfiguration;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAInboundConnectionType;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAOutboundConnectionConfiguration;
import com.ibm.j2ca.vta.emd.discovery.connection.VTAOutboundConnectionType;


/**
 * This is the main entry point for invocation of EMD. It should implement the constructor and call super constructor and pass the bundle name.
 * The bundle name is the name of the resource bundle that has to be used for property group or properties of the EMD.  
 * Set up the adapter type here and work from there.
 */
public class VTAMetadataDiscovery extends WBIMetadataDiscoveryImpl {
   // private static final String CURRENT_PACKAGE = "com.ibm.j2ca.vta.emd";
   // private static final String EMD_PACKAGE = "com.ibm.j2ca.vta.emd";
    private static final String CLASSNAME = "VTAMetadataDiscovery";
    
    /* generated by WAT
    private static WBIAdapterTypeImpl adapterType;
    WBIMetadataConnectionImpl connection;
    WBIOutboundConnectionTypeImpl connType;
    */
    VTAMetadataTree tree;

    //added by VTADev team
	WBIAdapterTypeImpl adapterType;// = null;
	WBIMetadataConnectionImpl connection;
	private static WBIOutboundConnectionTypeImpl connType;
	PropertyNameHelper helper = null;

    /**
     * Create a new instance of VTAMetadataDiscovery
     */
    public VTAMetadataDiscovery() throws MetadataException {
        /*
        super(CURRENT_PACKAGE);
        adapterType = new VTAAdapterType();
        */
    	//added by VTADev team
    	super("com.ibm.j2ca.vta.emd");
		helper = new PropertyNameHelper(null, "com.ibm.j2ca.vta.emd", null);
    }

    /**
     * It will return a list of adapter type supported by EMD implementation. For JCA adapters, there is only one adapter type.
     * @return	the value of adapterType
     * @see 	commonj.connector.metadata.discovery.MetadataDiscovery#getAdapterTypeSummaries()
     */
    public AdapterTypeSummary[] getAdapterTypeSummaries() throws MetadataException {
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataDiscovery", "getAdapterTypeSummaries");

		adapterType = new WBIAdapterTypeImpl("com.ibm.j2ca.vta.VTAResourceAdapter", 2, 1);
		adapterType.setDescription("VTA");
		adapterType.setDisplayName("VTA");
		adapterType.setId("VTA");
		adapterType.setVendor("IBM");
		adapterType.setVersion("6.1");
	    
		VTAOutboundConnectionType outConnTypeForMetadata = new VTAOutboundConnectionType(adapterType, helper);
		outConnTypeForMetadata.setIsSupportedInMetadataService(true);
		outConnTypeForMetadata.setIsSupportedAtRuntime(false);
		outConnTypeForMetadata.setManagedConnectionFactoryJavaBean("com.ibm.j2ca.vta.outbound.VTAManagedConnectionFactory"); //$NON-NLS-1$
		outConnTypeForMetadata.setResourceAdapterJavaBean("com.ibm.j2ca.vta.VTAResourceAdapter"); //$NON-NLS-1$
		adapterType.addOutboundConnectionType(outConnTypeForMetadata);

		VTAOutboundConnectionType outConnTypeForRuntime = new VTAOutboundConnectionType(adapterType, helper);
		outConnTypeForRuntime.setIsSupportedInMetadataService(false);
		outConnTypeForRuntime.setIsSupportedAtRuntime(true);
		outConnTypeForRuntime.setManagedConnectionFactoryJavaBean("com.ibm.j2ca.vta.outbound.VTAManagedConnectionFactory"); //$NON-NLS-1$
		outConnTypeForRuntime.setResourceAdapterJavaBean("com.ibm.j2ca.vta.VTAResourceAdapter"); //$NON-NLS-1$
		adapterType.addOutboundConnectionType(outConnTypeForRuntime);

		VTAInboundConnectionType inConnTypeForRuntime = new VTAInboundConnectionType(adapterType, helper);
		inConnTypeForRuntime.setResourceAdapterJavaBean("com.ibm.j2ca.vta.VTAResourceAdapter"); //$NON-NLS-1$
		inConnTypeForRuntime.setActivationSpecJavaBean("com.ibm.j2ca.vta.inbound.VTAActivationSpecWithXid"); //$NON-NLS-1$
		adapterType.addInboundConnectionType(inConnTypeForRuntime);

		return new WBIAdapterTypeImpl[] { adapterType };

    }

    /**
     * This method returns the instance of adapterType for a given ID. If the
     * enterprise metadata discovery implementation supports only one adapterType,
     * then it returns either that ID or the input ID.
     *
     * @return 	an instance of adapterType
     * @see commonj.connector.metadata.discovery.MetadataDiscovery#getAdapterType(java.lang.String)
     */
    public AdapterType getAdapterType(String adapterTypeID) throws MetadataException {
    	/* generated by WAT
    	getLogUtils().traceMethodEntrance(CLASSNAME, "getAdapterType()"); 	
        if (adapterTypeID.equals("VTA Resource Adapter")) {
            return adapterType;
        } else
            throw new MetadataException("Wrong adapterTypeID, 'VTA ResourceAdapter' expected, received: " + adapterTypeID);
        */
    	//added by VTADev team
    	if (adapterType == null)
			this.getAdapterTypeSummaries();
		return adapterType;
		
    }

    /**
     * This method returns an instance of the WBIMetadataTreeImpl implementation.
     * Each enterprise metadata discovery implementation extends the WBIMetadataTreeImpl
     * class and an instance of that class is returned from this method.
     *
     * @return an instance of MetadataTree
     * @see commonj.connector.metadata.discovery.MetadataDiscovery#getMetadataTree(commonj.connector.metadata.discovery.connection.MetadataConnection)
     */
    public MetadataTree getMetadataTree(MetadataConnection conn) throws MetadataException {
    	if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataDiscovery", "getMetadataTree");

		this.connection = (WBIMetadataConnectionImpl) conn;
		connType = (WBIOutboundConnectionTypeImpl) connection.getConnectionType();

		VTAMetadataTree tree = new VTAMetadataTree(connection, helper);
		tree.setSelectionStyle(MetadataTree.MULTI_SELECT);

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataDiscovery", "getMetadataTree");

		return tree;
    }

    /**
     * This class returns an instance of the inbound or outbound service description,
     * depending on which input selection is set. 
     *
     * @return the value of service description
     * @see commonj.connector.metadata.discovery.MetadataDiscovery#createServiceDescription(commonj.connector.metadata.discovery.connection.MetadataConnection,
     *      java.lang.String, java.lang.String,
     *      commonj.connector.metadata.discovery.MetadataSelection,
     *      commonj.connector.metadata.discovery.ToolEnvironmentContext)
     */
    public ServiceDescription createServiceDescription(MetadataSelection importSelection) throws MetadataException {
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataDiscovery", "createServiceDescription");

		ServiceDescription description = null;
		MetadataImportConfiguration[] confArray = importSelection.getSelection();
		if (confArray.length == 0)
			return description;

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Number of MetadataImportConfigurations " + confArray.length);

		String serviceType = ((VTAMetadataSelection) importSelection).getSelectedServiceType();

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Selected Service Type is " + serviceType);

		boolean inbound = serviceType.equalsIgnoreCase(MetadataConfigurationType.INBOUND_SERVICE.toString());

		//PropertyGroup pg = WBIMetadataSelectionImpl.getAppliedSelectionProperties();
		//SingleValuedProperty prop = (SingleValuedProperty) pg.getProperty("NameSpace");
		//String namespaceName = (String) prop.getValue();

		if (inbound) {
			VTAInboundServiceDescription iDesc = new VTAInboundServiceDescription(helper);
			String serviceName = "VTAInboundInterface"; //$NON-NLS-1$ 
			iDesc.setName(serviceName);
			// iDesc.setNameSpace(namespaceName);

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Setting function Descriptions");

			iDesc.setFunctionDescriptions(importSelection);
			WBIInboundConnectionConfigurationImpl inboundConf = (WBIInboundConnectionConfigurationImpl) adapterType.getInboundConnectionTypes()[0].createInboundConnectionConfiguration();
			// Set the BONameSpace as the one that is preselected
			inboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
			iDesc.setInboundConnectionAdvancedConfiguration(inboundConf);
			iDesc.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
			iDesc.setComment("Service Description for Inbound");
			iDesc.setMetadataSelectionProperties(WBIMetadataSelectionImpl.getAppliedSelectionProperties());
			description = iDesc;

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Inbound Service Description Created Successfully");

		} else {
			VTAOutboundServiceDescription oDesc = new VTAOutboundServiceDescription(helper);
			String serviceName = "VTAOutboundInterface";
			oDesc.setName(serviceName);
			// oDesc.setNameSpace(namespaceName);

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Setting function Descriptions");

			oDesc.setFunctionDescriptions(importSelection);
			oDesc.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
			oDesc.setComment("Service Description for Outbound");
			oDesc.setMetadataSelectionProperties(WBIMetadataSelectionImpl.getAppliedSelectionProperties());
			VTAOutboundConnectionConfiguration outboundConf = (VTAOutboundConnectionConfiguration) adapterType.getOutboundConnectionTypes()[1].createOutboundConnectionConfiguration();
			outboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
			oDesc.setOutboundConnectionAdvancedConfiguration(outboundConf);
			description = oDesc;

			if (helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.FINER, "VTAMetadataDiscovery", "createServiceDescription", "Oubound Service Description Created Successfully");
		}

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataDiscovery", "createServiceDescription");

		return description;
    }

    public void setToolContext(ToolContext toolContext) {
    	/*WAT generated code
        super.setToolContext(toolContext);
        try {
            LogUtils logUtils = new LogUtils(toolContext.getLogger(), "com.ibm.j2ca.vta.emd", "VTA Adapter", "6.2.0.0"); // TODO update the version
            if(logUtils != null) logUtils.trace(Level.FINER, CLASSNAME, "getToolContext()", "LogUtils Initilialized");
            super.setLogUtils(logUtils);
        } catch (ResourceAdapterInternalException e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            System.out.println("Unable to create LogUtils instance");
        }*/
    	//added by VTADev team
		super.setToolContext(toolContext);
		try {
			AdapterType adapterType = new WBIAdapterTypeImpl("com.ibm.j2ca.vta.VTAResourceAdapter", 2, 1);
			LogUtils logUtils = new LogUtils(toolContext.getLogger(), "com.ibm.j2ca.vta.emd", adapterType.getId(), adapterType.getVersion());
			LogRecord record = new LogRecord(Level.FINEST, "Entering setToolContext"); //$NON-NLS-1$ 
			toolContext.getLogger().log(record);
			helper.setLogUtils(logUtils);
			helper.setToolContext(toolContext);
			super.setLogUtils(logUtils);
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}

    }

    public static MetadataConfigurationType[] getMetadataConfiguration() {
		MetadataConfigurationType[] types = new MetadataConfigurationType[5];
    
		types[0] = MetadataConfigurationType.GENERATED_DATA_BINDING;
		types[1] = MetadataConfigurationType.INBOUND_SERVICE;
		types[2] = MetadataConfigurationType.OUTBOUND_SERVICE;
		types[3] = MetadataConfigurationType.GENERATED_RECORDS;
		types[4] = MetadataConfigurationType.GENERIC_RECORDS;
		
		return types;
    }
    
    public MetadataConfigurationType[] getSupportedConfiguration() {

		MetadataConfigurationType[] array = new MetadataConfigurationType[5];
		array[0] = MetadataConfigurationType.GENERATED_DATA_BINDING;
		array[1] = MetadataConfigurationType.GENERIC_RECORDS;
		array[2] = MetadataConfigurationType.GENERATED_RECORDS;
		array[3] = MetadataConfigurationType.INBOUND_SERVICE;
		array[4] = MetadataConfigurationType.OUTBOUND_SERVICE;
		return array;

		//return getMetadataConfiguration();
	}
	
	 /**
     * @param importSelection
     * @param selection
     * @param nameSpace
     * @return
     * @throws MetadataException
     */
    private ServiceDescription createOutboundServiceDescription(MetadataSelection importSelection, WBIMetadataSelectionImpl selection, String nameSpace) throws MetadataException {
        ServiceDescription description;
        VTAOutboundServiceDescription outboundDescription = new VTAOutboundServiceDescription();
        String interfaceName = "TwineBallOutboundInterface";
        outboundDescription.setName(interfaceName);
        outboundDescription.setNameSpace(nameSpace);
        getLogUtils().trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting function Descriptions");
        outboundDescription.setFunctionDescriptions(importSelection);
        getLogUtils().trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting MetadataConnectionConfiguration");
        outboundDescription.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
        outboundDescription.setComment("Service Description for Outbound");
        outboundDescription.setMetadataSelectionProperties(selection.getAppliedSelectionProperties());
        WBIOutboundConnectionConfigurationImpl outboundConf = (WBIOutboundConnectionConfigurationImpl) adapterType.getOutboundConnectionTypes()[1].createOutboundConnectionConfiguration();
        WBIManagedConnectionFactory mcf = (WBIManagedConnectionFactory)connType.getManagedConnectionFactoryJavaBean();
        mcf.set_LogUtils(null);
        outboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
        description = outboundDescription;
        outboundDescription.setOutboundConnectionAdvancedConfiguration(outboundConf);
        getLogUtils().trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Service Description Created Successfully");
        return description;
    }


    /**
     * @param importSelection
     * @param selection
     * @param nameSpace
     * @return
     * @throws MetadataException
     */
    private ServiceDescription createInboundServiceDescription(MetadataSelection importSelection, WBIMetadataSelectionImpl selection, String nameSpace) throws MetadataException {
        VTAInboundServiceDescription inboundDescription = new VTAInboundServiceDescription();
        String serviceName = "SampleInboundInterface";
        inboundDescription.setName(serviceName);
        inboundDescription.setNameSpace(nameSpace);
        super.getLogUtils().trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting function Descriptions");
        inboundDescription.setFunctionDescriptions(importSelection);
        WBIInboundConnectionConfigurationImpl inboundConf = (WBIInboundConnectionConfigurationImpl) adapterType.getInboundConnectionTypes()[0].createInboundConnectionConfiguration();
        inboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
        super.getLogUtils().trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting InboundAdvancedConnectionConfiguration");
        inboundDescription.setInboundConnectionAdvancedConfiguration(inboundConf);
        super.getLogUtils().trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting MetadataConnectionConfiguration");
        inboundDescription.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
        inboundDescription.setComment("Service Description for Inbound");
        inboundDescription.setMetadataSelectionProperties(selection.getAppliedSelectionProperties());
        super.getLogUtils().trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Service Description Created Successfully");
        return inboundDescription;
    }

    /**
    * @param serviceType
    * @return
    */
   private String getDirectionality(PropertyGroup serviceType) {

       SingleValuedProperty prop = (SingleValuedProperty) serviceType.getProperty("ServiceType");
       String directionality = (String) prop.getValue();

       return directionality;
   }

   /**
    * @param serviceType
    * @return
    */
   private String getNameSpace(PropertyGroup serviceType) {
       SingleValuedProperty prop;
       prop = (SingleValuedProperty) serviceType.getProperty("NameSpace");
       String nameSpace = (String) prop.getValue();
       if (nameSpace == null) nameSpace = Constants.TB_DEFAULT_NAMESPACE;
       return nameSpace;
   }
}