/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
 
package com.ibm.j2ca.vta.emd.discovery;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;

import com.ibm.j2ca.base.xsdutil.BOType;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataObjectImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataObjectResponseImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.MetadataImportConfiguration;
import commonj.connector.metadata.discovery.MetadataObjectResponse;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This class can retrieve the object information in each EIS node, and store it for later use.
 * The MetadataObject represents the node of the MetadataTree. You define properties
 * and filters in this object that help guide enterprise metadata discovery.  The object
 * information retrieved and be stored for later use.
 */
public class VTAMetadataObject extends WBIMetadataObjectImpl {
    private static final String CLASSNAME = "VTAMetadataObject";

    //added by VTADev Team
	private VTAMetadataTree tree;
	private ArrayList childList;
	String content;
	String targetNameSpace;
	String header;
	String charSet;
	BOType boType;
	String aliasTag;
	String xsdName;
	boolean containedChild;
	boolean partOfSameXSD;
	PropertyNameHelper helper=null;

   /**
     * Set up the object's properties right in the constructor.   
     */
    VTAMetadataObject() throws MetadataException{
        super();
        // TODO Auto-generated constructor stub
    }
    
    //added by VTADev Team
    public VTAMetadataObject(VTAMetadataTree tree,PropertyNameHelper helper) {
		this.tree = tree;
		this.helper = helper;
	}

    /**
     * @return
     * @see commonj.connector.metadata.discovery.MetadataObject#isSelectableForImport()
     */
  /*  public boolean isSelectableForImport() {
    	getLogUtils().traceMethodEntrance(CLASSNAME, "isSelectableForImport()"); 
        return true;
    }*/

    /**
     * This method creates filtering properties that make fetching of child objects
     * more efficient and usable.
     *
     * @return	PropertyGroup value
     * @see commonj.connector.metadata.discovery.MetadataObject#createFilteringProperties()
     */
    public PropertyGroup createFilteringProperties() {
   		//getLogUtils().traceMethodEntrance(CLASSNAME, "createFilteringProperties()"); 
        return null;
    }

    /**
     * Displays help text on selected object's description dialog
     * This method should return a property group that gives more information about the specific object in MetadataTree 
     * which provides information to the user about the metadatobject instance.
     * 
     * @return	propertyGroup value
     * @see commonj.connector.metadata.discovery.MetadataObject#getObjectProperties()
     */
    public PropertyGroup getObjectProperties() {
    	WBIPropertyGroupImpl pg = null;
		return pg;
    }

    /**
     * This method should return an instance of WBIMetadataObjectResponseImpl populated with child MetadataObjects.
     * @return	a metadatObjectResponse object representing the child objects	
     * @see commonj.connector.metadata.discovery.MetadataObject#getChildren(commonj.connector.metadata.discovery.properties.PropertyGroup,
     *      commonj.connector.metadata.discovery.ToolEnvironmentContext)
     */
    public MetadataObjectResponse getChildren(PropertyGroup filterParameters) throws MetadataException {
		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAMetadataObject", "getChildren");
		
		WBIMetadataObjectResponseImpl response = new WBIMetadataObjectResponseImpl();
		ArrayList list = new ArrayList();
		try {
			if (boType != null) {
				List children = boType.getChildren();
				if (children != null)
					for (Iterator e = children.iterator(); e.hasNext();) {
						BOType child = (BOType) e.next();
						String name = child.getName();
						// Create the child MetadataObject
						VTAMetadataObject childObject = new VTAMetadataObject(tree,helper);
						childObject.setTargetNameSpace(child.getType().getNameSpace());
						childObject.setDisplayName(name);
						childObject.setBOType(child);
						if (child.getChildren() != null && child.getChildren().size() > 0) {
							childObject.setHasChildren(true);
						}
						// Mark these as not selectable
						childObject.setSelectableForImport(false);
						childObject.setContainedChild(true);
						//childObject.setContentType(getContentType());
						childObject.setBOName(name);
						
						// Set the parent object
						childObject.setParent(this);
						childObject.setLocation(childObject.getTargetNameSpace() + childObject.getBOName());
						
						// Copy the content and header from the
						// object created for the same complexType
						// as a top level object. This would have happened
						// in the step of DTFMetadataTree.listMetadataObjects()
						VTAMetadataObject topObject = (VTAMetadataObject) tree.getMetadataObject(childObject.getLocation());
						childObject.setHeader(topObject.getHeader());
						childObject.setAliasTag(topObject.getAliasTag());
						childObject.setContent(topObject.getContent());
						childObject.setXsdName(topObject.getXsdName());
						if (childObject.getXsdName().equals(this.getXsdName())) {
							childObject.setPartOfSameXSD(true);
							topObject.setPartOfSameXSD(true);
						}
						list.add(childObject);
					}
			} else {
				if (childList != null)
					list = childList;
			}
		} catch (Exception e) {
			throw new MetadataException(e.getMessage());
		}
		response.setObjects(list);

		if (helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAMetadataObject", "getChildren");

		return response;
    }

    //the following methods have been added by VTADev Team
    public MetadataImportConfiguration createImportConfiguration() throws MetadataException {
		VTAMetadataImportConfiguration conf = new VTAMetadataImportConfiguration(this);
		conf.setLocation(this.getLocation());
		this.setMetadataImportConfiguration(conf);
		return conf;

	}

	protected void setContent(String content) {
		this.content = content;
	}

	public String getContent() {
		return content;
	}

	public void setTargetNameSpace(String targetNameSpace) {
		this.targetNameSpace = targetNameSpace;
	}

	public String getTargetNameSpace() {
		return this.targetNameSpace;
	}

	public void setHeader(String header) {
		this.header = header;
	}

	public String getHeader() {
		return header;
	}

	public void setCharSet(String charSet) {
		this.charSet = charSet;
	}

	public String getCharSet() {
		return charSet;
	}

	public BOType getBOType() {
		return boType;
	}

	public void setBOType(BOType boType) {
		this.boType = boType;
	}

	public String getAliasTag() {
		return aliasTag;
	}

	public void setAliasTag(String aliasTag) {
		this.aliasTag = aliasTag;
	}

	public String getXsdName() {
		return xsdName;
	}

	public void setXsdName(String xsdName) {
		this.xsdName = xsdName;
	}
	public void setContainedChild (boolean containedChild) {
		this.containedChild = containedChild;
	}
	
	public boolean getContainedChild() {
		return this.containedChild;
	}
	
	public boolean isPartOfSameXSD() {
		return partOfSameXSD;
	}

	public void setPartOfSameXSD(boolean partOfSameXSD) {
		this.partOfSameXSD = partOfSameXSD;
	}
 }