
/*
 * IBM Confidential
 * OCO Source Materials
 * (C) Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.emd.discovery;

import java.util.logging.Level;

import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataSelectionImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIMultiValuedPropertyImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import com.ibm.j2ca.vta.emd.Constants;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyGroup;



/**
 * This method represents the object that holds the MetadataObjects that users
 * can select for importing. This class also holds properties that users specify
 * to select from the set of available MetadataObjects.
 */
public class VTAMetadataSelection extends WBIMetadataSelectionImpl {
    private static final String CLASSNAME = "VTAMetadataSelection";
    public static final String RELATIVEPATH = "RelativePath";
    public static final String MAXRECORDS = "MaxRecords";

    //added by VTADev team
	private String artifactType = null;
	private String serviceType = null;
	private String[] operationsWPS = {WBIInteractionSpec.CREATE_OP,WBIInteractionSpec.UPDATE_OP,WBIInteractionSpec.DELETE_OP,
			WBIInteractionSpec.RETRIEVE_OP,WBIInteractionSpec.APPLY_CHANGES_OP};
	private String[] operationsNonWPS = {WBIInteractionSpec.CREATE_OP,WBIInteractionSpec.UPDATE_OP,WBIInteractionSpec.DELETE_OP,
			WBIInteractionSpec.RETRIEVE_OP};
	
	/**
	 * constructor
	 */
	public VTAMetadataSelection() {
		//added by VTADev team
		try {
			applySelectionProperties(null);
		} catch (Exception e) {
		}
		//super();
	}

	/**
	 * This method returns a property group that is used to capture inputs from
	 * users. These inputs include business object namespace, supported operations,
	 * and the relative path in the module project where XML schema definitions
	 * should be saved.
	 *
	 * @return 	propertyGroup instance
	 * @see commonj.connector.metadata.discovery.MetadataSelection#createSelectionProperties()
	 */
	public PropertyGroup createSelectionProperties() {
		/* TODO Generate method
		getLogUtils().traceMethodEntrance(CLASSNAME, "getAdapterTypeSummaries()"); 
		return null;
		*/
		VTAMetadataDiscovery.getLogUtils().traceMethodEntrance("VTAMetadataSelection", "createSelectionProperties");
		WBIPropertyGroupImpl pg = null;
		try {
			pg = new WBIPropertyGroupImpl("Selection Properties");
			pg.setDisplayName("SelectionProperties"); 
			pg.setDescription("SelectionProperties"); 
			
			MetadataConfigurationType[] types = WBIMetadataDiscoveryImpl.getMetadataConfiguration();

			for(int i=0;i<types.length;i++){
				if (types[i].equals(MetadataConfigurationType.GENERIC_RECORDS) || types[i].equals(MetadataConfigurationType.GENERATED_RECORDS)
						|| types[i].equals(MetadataConfigurationType.GENERATED_DATA_BINDING) || types[i].equals(MetadataConfigurationType.GENERIC_DATA_BINDING)) {
					artifactType = types[i].toString();
				}
				if(types[i].equals(MetadataConfigurationType.INBOUND_SERVICE) || types[i].equals(MetadataConfigurationType.OUTBOUND_SERVICE)){
					serviceType = types[i].toString();
				}
			}

			WBIMultiValuedPropertyImpl operationProp = new WBIMultiValuedPropertyImpl("Operations", String.class); //$NON-NLS-1$
			operationProp.setDisplayName("Operations"); 
			operationProp.setDescription("Operations");  
			operationProp.setRequired(true);
			String[] operations=null;
			if(serviceType.equals(MetadataConfigurationType.INBOUND_SERVICE.toString())){
				operations = new String[3];
				operations[0] ="Create";
				operations[1] ="Update";
				operations[2] ="Delete"; 
			}else{
				if(artifactType.equals(MetadataConfigurationType.GENERATED_DATA_BINDING.toString())){// is WPS
					operations = operationsWPS;
				}else{
					operations = operationsNonWPS;
				}			
			}
			operationProp.setValidValues(operations);
			for(int idx=0;idx<operations.length;idx++){
				operationProp.addValue(operations[idx]);
			}
			pg.addProperty(operationProp);

			//commit it out.
//			WBISingleValuedPropertyImpl nameSpaceProp = new WBISingleValuedPropertyImpl("NameSpace", String.class);
//			nameSpaceProp.setValue("http://www.ibm.com/xmlns/prod/websphere/j2ca/vta"); 
//			pg.addProperty(nameSpaceProp);
//			nameSpaceProp.setDisplayName("NameSpace"); 
//			nameSpaceProp.setDescription("NameSpace"); 
//			nameSpaceProp.setRequired(true);

			//Copy the applied properties to the new instance
			if (getAppliedSelectionProperties() != null)
				EMDUtil.copyValues(getAppliedSelectionProperties(), pg);
		
		} catch (MetadataException e) {
			WBIMetadataDiscoveryImpl.getLogUtils().trace(Level.SEVERE, "VTAMetadataSelection", "createSelectionProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage());
		}
		VTAMetadataDiscovery.getLogUtils().traceMethodExit("VTAMetadataSelection", "createSelectionProperties");
		return pg;
	}

    /**
     * @param propertyGroup
     * @throws MetadataException
     */
    private void createRelativePathProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
        WBISingleValuedPropertyImpl location = new WBISingleValuedPropertyImpl(RELATIVEPATH, String.class);
        location.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(RELATIVEPATH));
        location.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(RELATIVEPATH));
        propertyGroup.addProperty(location);
    }

	/**
	 * @return namespace
	 */
    public static String getNamespace() {
        PropertyGroup pg = getAppliedSelectionProperties();
        if (pg != null) {
            WBISingleValuedPropertyImpl namespaceProp = (WBISingleValuedPropertyImpl) pg.getProperty("NameSpace"); //$NON-NLS-1$
            if (namespaceProp.getValue() != null)
                return (String) namespaceProp.getValue();
        }
        return Constants.TB_DEFAULT_NAMESPACE;
    }

    //the following methods have been added by VTADev team
	public String getSelectedArtifactType(){
		return this.artifactType;
	}
	public String getSelectedServiceType(){
		return this.serviceType;
	}

}