/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.emd.discovery.connection;

import com.ibm.j2ca.vta.VTAResourceAdapter;
import com.ibm.j2ca.vta.emd.discovery.VTAMetadataSelection;
import com.ibm.j2ca.vta.inbound.VTAActivationSpecWithXid;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;

import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;

import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

//added by VTADev team
import java.util.logging.Level;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import commonj.connector.metadata.discovery.properties.PropertyDescriptor;

/**
 * This class is used to specify inbound connection configuration properties,
 * including those for ActivationSpec for your enterprise metadata discovery
 * implementation.
 */
public class VTAInboundConnectionConfiguration extends
		WBIInboundConnectionConfigurationImpl {
	//added by VTADev team
	PropertyNameHelper helper = null;
	
	public VTAInboundConnectionConfiguration(WBIInboundConnectionTypeImpl connType, PropertyNameHelper helper) throws MetadataException {
		super(connType);
		this.helper = helper;
	}

	/**
	 * @param 	connType
	 * @throws 	MetadataException
	 */
	public VTAInboundConnectionConfiguration(WBIInboundConnectionTypeImpl connType)
			throws MetadataException {
		super(connType);
	}

	/**
	 * This method returns an instance of property group that represents ActivationSpec
	 * and ResourceAdapter properties together in a consolidated property group.
	 * Property groups required for discovery might differ from those needed at
	 * run time. WebSphere Integration Developer uses this method to display the
	 * first connection configuration screen for enterprise metadata discovery.
	 *
	 * @return	PropertyGroup instance
	 * @see 	commonj.connector.metadata.discovery.connection.ConnectionConfiguration#createUnifiedProperties()
	 */
	public PropertyGroup createUnifiedProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAInboundConnectionConfiguration", "createUnifiedProperties");

		WBIPropertyGroupImpl propGroup = null;
		try {
			propGroup = (WBIPropertyGroupImpl) createActivationSpecProperties();
			WBIPropertyGroupImpl adapterProp = (WBIPropertyGroupImpl) createResourceAdapterProperties();
			WBISingleValuedPropertyImpl logProp = (WBISingleValuedPropertyImpl) adapterProp.getProperties()[0];
			if (logProp != null) {
				if (logProp != null)
					logProp.setValue("VisualTestAdapter");
			}
			if (adapterProp != null) {
				PropertyDescriptor[] properties = adapterProp.getProperties();
				for (int i=0; i< adapterProp.getProperties().length; i++)
					propGroup.addProperty(properties[i]);
			}
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAInboundConnectionConfiguration", "createUnifiedProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}
		
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAInboundConnectionConfiguration", "createUnifiedProperties");

		return propGroup;
	}

	/**
	 * This method returns an instance of PropertyGroup that represents properties 
	 * that you can configure for the ActivationSpec bean.
	 *
	 * @return 	propertyGroup
	 * @see 	commonj.connector.metadata.discovery.connection.InboundConnectionConfiguration#createActivationSpecProperties()
	 */
	public PropertyGroup createActivationSpecProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAInboundConnectionConfiguration", "createActivationSpecProperties");

		WBIPropertyGroupImpl connProp = null;
		try {
			
			EMDUtil util = new EMDUtil();
			connProp = (WBIPropertyGroupImpl) util.getPropertyGroup(new VTAActivationSpecWithXid(), helper);
			WBISingleValuedPropertyImpl namespaceProp = (WBISingleValuedPropertyImpl) connProp.getProperty("BONamespace");
			if (namespaceProp != null)
				connProp.remove(namespaceProp);
			
			WBIPropertyGroupImpl machineCredPG = new WBIPropertyGroupImpl("MachineCredentials", helper);
			machineCredPG.setDisplayName(helper.getPropertyName("MachineCredentials"));
			machineCredPG.setDescription(helper.getPropertyDescription("MachineCredentials")); 

			WBISingleValuedPropertyImpl prop = new WBISingleValuedPropertyImpl("Hostname", String.class);  
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("HostName"));  
			prop.setDescription(helper.getPropertyDescription("HostName"));
			prop.setValue("localhost");
			machineCredPG.addProperty(prop);

			prop = new WBISingleValuedPropertyImpl("Portnumber", String.class);  
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("Port"));  
			prop.setDescription(helper.getPropertyDescription("Port"));
			prop.setValue("9000");
			machineCredPG.addProperty(prop);
			
			connProp.addProperty(machineCredPG);

			if (getAppliedProperties() != null)
				util.copyValues(getAppliedProperties(), connProp,helper);
			
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAInboundConnectionConfiguration", "createActivationSpecProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}

		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAInboundConnectionConfiguration", "createActivationSpecProperties");

		return connProp;
	}

	/** 
	 * This method returns an instance of PropertyGroup that represents properties
	 * you can configure for the ResourceAdapter bean.
	 * The method getPropertyGroup() in EMDUtil class can be used to fetch the properties for base class WBIResourceAdapter bean.
	 * @return 	adapterType
	 * @see 	commonj.connector.metadata.discovery.connection.ConnectionConfiguration#createResourceAdapterProperties()
	 */
	public PropertyGroup createResourceAdapterProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAInboundConnectionConfiguration", "createResourceAdapterProperties");

		WBIPropertyGroupImpl propGroup = null;
		try {
			VTAResourceAdapter ra = new VTAResourceAdapter();
			EMDUtil util = new EMDUtil();
			propGroup = (WBIPropertyGroupImpl) util.getPropertyGroup(ra, helper);

			propGroup.setExpert(true);
			if (getAppliedProperties() != null && propGroup != null)
				EMDUtil.copyValues(getAppliedProperties(), propGroup);
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAInboundConnectionConfiguration", "createResourceAdapterProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}

		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAInboundConnectionConfiguration", "createResourceAdapterProperties");
		return propGroup;
	}

}