/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.emd.discovery.connection;

import com.ibm.j2ca.extension.emd.discovery.WBIAdapterTypeImpl;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIInboundConnectionTypeImpl;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.discovery.connection.InboundConnectionConfiguration;

//added by VTADev team
import java.util.logging.Level;

import com.ibm.j2ca.extension.emd.PropertyNameHelper;

/**
 * This is the connectionType for inbound. It is used to create Inbound service
 * description. An adapter can support multiple Inbound connection types, the
 * mapping of inbound connection is to Activation Specs, as many activation specs
 * an adapter supports than many inbound connection types would have to be supported.
 */
public class VTAInboundConnectionType extends WBIInboundConnectionTypeImpl {
   	
   	VTAInboundConnectionConfiguration conf = null;
   	
   	//added by VTADev team
	PropertyNameHelper helper = null;

	public VTAInboundConnectionType(WBIAdapterTypeImpl adapterType,PropertyNameHelper helper) throws MetadataException {
		super(adapterType);
		this.helper = helper;
	}

   	/**
	 * Creates an instance of VTAInboundConnectionType
	 *
	 * @param adapterType
	 * @throws commonj.connector.metadata.MetadataException
	 */
	public VTAInboundConnectionType(WBIAdapterTypeImpl adapterType) throws MetadataException {
		super(adapterType);
        setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription("ConnectionType"));
        setId("VTA");
        setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName("ConnectionType"));
	}

	/**
	 *
	 * @return InboundConnectionConfiguration instance
	 * @see commonj.connector.metadata.discovery.connection.InboundConnectionType#createInboundConnectionConfiguration()
	 */
	public InboundConnectionConfiguration createInboundConnectionConfiguration() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAInboundConnectionType", "createInboundConnectionConfiguration");
		
		try {
			if (conf == null)
				conf = new VTAInboundConnectionConfiguration(this,helper);
		} catch (Exception e) {
			//if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				//helper.getLogUtils().log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, "VTAInboundConnectionType", "createInboundConnectionConfiguration", "1004", new Object[] { e.getMessage() }); //$NON-NLS-1$ 
			throw new RuntimeException(e.getMessage(), e);
		}

		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAInboundConnectionType", "createInboundConnectionConfiguration");
		return conf;
	}
}