/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.emd.discovery.connection;

import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionConfigurationImpl;
import com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionTypeImpl;

import commonj.connector.metadata.discovery.properties.PropertyGroup;
import com.ibm.j2ca.extension.emd.EMDUtil;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBIPropertyGroupImpl;
import com.ibm.j2ca.extension.emd.discovery.properties.WBISingleValuedPropertyImpl;
import commonj.connector.metadata.MetadataException;
import com.ibm.j2ca.vta.VTAResourceAdapter;

//added by VTADev team
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.vta.VTAConstants;
import java.util.logging.Level;
import com.ibm.j2ca.extension.emd.discovery.properties.extensions.WBIFolderProperty;


/**
 * This class is used to specify outbound connection configuration properties,
 * including those for ManagedConnectionFactory for your enterprise metadata discovery
 * implementation. It will be used to connect to EIS for discovery and to create service descriptions.
 */
public class VTAOutboundConnectionConfiguration extends
		WBIOutboundConnectionConfigurationImpl {
	private PropertyNameHelper helper = null;
	
	public VTAOutboundConnectionConfiguration(WBIOutboundConnectionTypeImpl connType,PropertyNameHelper helper) throws MetadataException {
		super(connType, helper);
		this.helper = helper;	
	}

	
	/**
	 * @param 	connType
	 * @throws 	MetadataException
	 */
	public VTAOutboundConnectionConfiguration(
			com.ibm.j2ca.extension.emd.discovery.connection.WBIOutboundConnectionTypeImpl connType)
			throws commonj.connector.metadata.MetadataException {
		super(connType);
	}

	/**
	 * This method returns an instance of property group that represents ManagedConnectionFactory
	 * and ResourceAdapter properties together in a consolidated property group.
	 * Property groups required for discovery might differ from those needed at
	 * run time. Accordingly, this method can check isSupportedInMetadataService
	 * and then create the corresponding property groups. WebSphere Integration
	 * Developer uses this method to display the first connection configuration
	 * screen for enterprise metadata discovery.
	 *
	 * @return	PropertyGroup instance
	 * @see commonj.connector.metadata.discovery.connection.ConnectionConfiguration#createUnifiedProperties()
	 */
	public PropertyGroup createUnifiedProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAOutboundConnectionConfiguration", "createUnifiedProperties");
		
		WBIPropertyGroupImpl propGroup = null;
		try {
			if (this.getOutboundConnectionType().isSupportedInMetadataService()) {

				propGroup = new WBIPropertyGroupImpl("Connection Properties");
				propGroup.setDisplayName(helper.getPropertyName("ConnectionProperties"));
				propGroup.setDescription(helper.getPropertyDescription("ConnectionProperties"));
				
				WBIFolderProperty propFolder = new WBIFolderProperty("XSDFolder", java.io.File.class);
				propFolder.setRequired(true);
				propFolder.setMustExist(true);
				propFolder.setDisplayName(helper.getPropertyName("XSDFolder"));
				propFolder.setDescription(helper.getPropertyDescription("XSDFolder"));
				
				propGroup.addProperty(propFolder);
				
			} else {
				propGroup = (WBIPropertyGroupImpl) createManagedConnectionFactoryProperties();
				
				PropertyGroup adapterProp = createResourceAdapterProperties();

				if(adapterProp != null){
					WBISingleValuedPropertyImpl logProp = (WBISingleValuedPropertyImpl) adapterProp.getProperties()[0];
					if (logProp != null)
					{
						logProp.setValue(VTAConstants.VTA_ADAPTERID);
					}
					propGroup.addProperty(adapterProp);
				}

			}
			if (this.getOutboundConnectionType().isSupportedInMetadataService() && (getAppliedProperties() != null)) {
				EMDUtil.copyValues(getAppliedProperties(), propGroup);
			}

		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAOutboundConnectionConfiguration", "createUnifiedProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}
		
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAOutboundConnectionConfiguration", "createUnifiedProperties");
		return propGroup;
	}

	/**
	 * This method returns an instance of PropertyGroup that represents properties 
	 * that you can configure for the ManagedConnectionFactory bean. The method getPropertyGroup() provided in EMDUtil class
	 * can be used to fetch the properties for base class WBIManagedConnectionFactory bean. You can add your own properties
	 * for these managed connection factory bean class.
	 *
	 * @return 	propertyGroup
	 * @see commonj.connector.metadata.discovery.connection.OutboundConnectionConfiguration#createManagedConnectionFactoryProperties()
	 */
	public PropertyGroup createManagedConnectionFactoryProperties() {
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodEntrance("VTAOutboundConnectionConfiguration", "createManagedConnectionFactoryProperties");

		WBIPropertyGroupImpl propGroup = null;
		try {
			propGroup = new WBIPropertyGroupImpl("Connection Properties");
			propGroup.setDisplayName(helper.getPropertyName("ConnectionProperties"));
			propGroup.setDescription(helper.getPropertyDescription("ConnectionProperties"));

			WBIPropertyGroupImpl mcPG = new WBIPropertyGroupImpl("MachineCredentials", helper); 
			mcPG.setDisplayName(helper.getPropertyName("MachineCredentials"));
			mcPG.setDescription(helper.getPropertyDescription("MachineCredentials")); 

			WBISingleValuedPropertyImpl prop = new WBISingleValuedPropertyImpl("Hostname", String.class);
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("HostName"));  
			prop.setDescription(helper.getPropertyDescription("HostName"));
			prop.setValue("localhost");
			mcPG.addProperty(prop);
			prop = new WBISingleValuedPropertyImpl("Portnumber", String.class);
			prop.setRequired(true);
			prop.setDisplayName(helper.getPropertyName("Port"));  
			prop.setDescription(helper.getPropertyDescription("Port"));
			prop.setValue("9000");
			mcPG.addProperty(prop);
			
			propGroup.addProperty(mcPG);

			
			if (getAppliedProperties() != null)
				EMDUtil.copyValues(getAppliedProperties(), propGroup);

		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAOutboundConnectionConfiguration", "createManagedConnectionFactoryProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}
		if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
			helper.getLogUtils().traceMethodExit("VTAOutboundConnectionConfiguration", "createManagedConnectionFactoryProperties");
		return propGroup;	}

	/** 
	 * This method returns an instance of PropertyGroup that represents properties that you can configure for the ResourceAdapter bean. 
	 * The method getPropertyGroup() in EMDUtil class can be used to fetch the properties for base class WBIResourceAdapter bean. 
	 * You can add your own properties for the resource adapter bean class.
	 *
	 * @return 	adapterType
	 * @see commonj.connector.metadata.discovery.connection.ConnectionConfiguration#createResourceAdapterProperties()
	 */
	public PropertyGroup createResourceAdapterProperties() {
		WBIPropertyGroupImpl propGroup = null;
		try {
			VTAResourceAdapter ra = new VTAResourceAdapter();
			EMDUtil util = new EMDUtil();
			propGroup = (WBIPropertyGroupImpl) util.getPropertyGroup(ra, helper);

			propGroup.setExpert(true);
			if (getAppliedProperties() != null && propGroup != null)
				EMDUtil.copyValues(getAppliedProperties(), propGroup);
		} catch (Exception e) {
			if ((helper.getLogUtils()!=null) && helper.getLogUtils().isTraceEnabled(Level.FINE))
				helper.getLogUtils().trace(Level.SEVERE, "VTAOutboundConnectionConfiguration", "createResourceAdapterProperties", "Error in creating properties ", e);
			throw new RuntimeException(e.getMessage(), e);
		}
		return propGroup;
	}
}