/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
 package com.ibm.j2ca.vta.emd.runtime;

import javax.resource.cci.Record;

import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.base.internal.WPSServiceHelper;
import com.ibm.j2ca.extension.dataexchange.sdo.DEFactorySDO;
import com.ibm.j2ca.vta.VTAConstants;
import com.ibm.j2ca.vta.VTAStructuredRecord;
import commonj.connector.runtime.DataBindingException;
import commonj.connector.runtime.RecordHolderDataBinding;
import commonj.sdo.DataObject;
import commonj.sdo.Property;

public abstract class VTADataBinding implements RecordHolderDataBinding {

	VTAStructuredRecord record = null;
	DataObject inputBG = null;
	DataObject container = null;

	
	/**
	 * This method should return the instance of record being build in
	 * setDataObject() call or passed through setRecord() call.
	 *
	 * @return	Record instance
	 * @throws 	DataBindingException
	 * @see commonj.connector.runtime.RecordHolderDataBinding#getRecord()
	 */
	public Record getRecord() throws DataBindingException {
		return record;
	}

	/**
	 * This method should hold the instance of the record passed to this method
	 * as a record instance for the binding implementation.
	 *
	 * @param	record
	 * @throws 	DataBindingException
	 * @see commonj.connector.runtime.RecordHolderDataBinding#setRecord(javax.resource.cci.Record)
	 */
	public void setRecord(Record record) throws DataBindingException {
		this.record = (VTAStructuredRecord) record;
	}

	/**
	 * This method should build an instance of SDO with the data that is returned
	 * from backend application. 
	 *
	 * @return	DataObject representing the returned data
	 * @throws 	DataBindingException
	 * @see commonj.connector.runtime.DataBinding#getDataObject()
	 */
	public DataObject getDataObject() throws DataBindingException {		
		DataObject returnValue = null;

		try {
			DEFactorySDO binding = new DEFactorySDO();
			DataObject dataObject = null;
			dataObject = WPSServiceHelper.createBusinessObject(this.getNamespaceURI(), this.getBusinessObjectName());
			//RetrieveAll, create a container SDO
			if (record.getOperationName().equalsIgnoreCase(WBIInteractionSpec.RETRIEVE_ALL_OP)) {
				DataObject returnObject = dataObject;
				Property p = (Property) returnObject.getType().getProperties().get(0);
				dataObject = WPSServiceHelper.createBusinessObject(p.getType().getURI(), p.getType().getName());
				boolean isBG = WPSServiceHelper.isBusinessGraph(returnObject);
				if (isBG)
					dataObject = dataObject.createDataObject(WPSServiceHelper.getRootBusinessObjectProperty(dataObject.getType()));
				Object[] array = new Object[1];
				array[0] = dataObject;
				record.initializeOutput(binding, array);
				while (record.getNext(true)) {
					DataObject object = (DataObject) binding.getBoundObject();
					if (dataObject.getContainer() != null) {
						DataObject BG = WPSServiceHelper.createDataObject(object.getType().getURI() + "bg", object.getType().getName() + "BG"); //$NON-NLS-1$ //$NON-NLS-2$
						BG.set(WPSServiceHelper.getRootBusinessObjectProperty(BG.getType()), object);
						BG.set(VTAConstants.VERB, record.getBGVerb());
						returnObject.getList(0).add(BG);
					} else
						returnObject.getList(0).add(object);
				}
				return returnObject;
			}

			boolean isBG = WPSServiceHelper.isBusinessGraph(dataObject);
			if (isBG)
				dataObject = dataObject.createDataObject(WPSServiceHelper.getRootBusinessObjectProperty(dataObject.getType()));
			Object[] array = new Object[1];
			array[0] = dataObject;
			record.initializeOutput(binding, array);
			if (record.getOperationName().equalsIgnoreCase(WBIInteractionSpec.DELETE_OP)) {
				if(record.getNext(true)){
				DataObject object = (DataObject) binding.getBoundObject();
				DataObject bg = dataObject.getContainer();
				if (bg != null) {
					returnValue = WPSServiceHelper.createDataObject(bg.getType().getURI(), bg.getType().getName());
					returnValue.set(WPSServiceHelper.getRootBusinessObjectProperty(returnValue.getType()), object);
					returnValue.set(VTAConstants.VERB, record.getBGVerb());
				} else{
					returnValue = object;
				}
				}
			} else if (record.getNext(true)) {
				DataObject object = (DataObject) binding.getBoundObject();
				DataObject bg = dataObject.getContainer();
				if (bg != null) {
					returnValue = WPSServiceHelper.createDataObject(bg.getType().getURI(), bg.getType().getName());
					returnValue.set(WPSServiceHelper.getRootBusinessObjectProperty(returnValue.getType()), object);
					returnValue.set(VTAConstants.VERB, record.getBGVerb());
				} else{
					returnValue = object;
				}
			}
		} catch (Exception e) {
			throw new DataBindingException("Failed in population return object", e); //$NON-NLS-1$
		}
		return returnValue;	}

	/**
	 * This method should build an instance of adapter record instance and initialize
	 * with the metadata that represents input SDO.
	 *
	 * @param	dataObj
	 * @throws 	DataBindingException
	 * @see 	commonj.connector.runtime.DataBinding#setDataObject(commonj.sdo.DataObject)
	 */
	public void setDataObject(DataObject dataObj) throws DataBindingException {
		try {
			record = new VTAStructuredRecord();
			inputBG = dataObj;
			DEFactorySDO binding = new DEFactorySDO();
			boolean isBG = WPSServiceHelper.isBusinessGraph(dataObj);
			DataObject dataObject = null;
			if (isBG) {
				dataObject = WPSServiceHelper.getRootBusinessObjectInstance(dataObj);
				String verb = dataObj.getString(VTAConstants.VERB);
				if(verb != null){
					record.setOperationName(verb);
					record.setBGVerb(verb);
				}
				record.setIsBG(true);
			} else
				dataObject = dataObj;
			binding.setBoundObject(dataObject);
			Object[] array = new Object[1];
			array[0] = dataObject;
			record.initializeInput(binding, array);
			record.setNamespace(inputBG.getType().getURI());
			record.setRecordName(inputBG.getType().getName());
		} catch (Exception e) {
			throw new DataBindingException("Failed to initialize cursor", e); //$NON-NLS-1$ 
		}
	}
	
	/**
	 * This method is used by DataBinding implementation in getDataObject() call
	 * to find out what is the SDO type that the binding should instantiate.
	 * The implementation for this method is provided by the generated databinding
	 * classes.
	 */
	public abstract String getNamespaceURI();

	/**
	 * This method is used by DataBinding implementation in getDataObject() call
	 * to find out what is the SDO type that the binding should instantiate.
	 * The implementation for this method is provided by the generated databinding
	 * classes.
	 */
	public abstract String getBusinessObjectName();

}
