/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.outbound;

import javax.resource.ResourceException;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;

import com.ibm.j2ca.base.WBIConnectionRequestInfo;
import com.ibm.j2ca.base.WBIManagedConnection;
import com.ibm.j2ca.base.WBIManagedConnectionFactory;
import com.ibm.j2ca.base.WBIManagedConnectionMetaData;
import com.ibm.j2ca.vta.VTAConstants;
import com.ibm.j2ca.base.WBILocalTransactionWrapper;
import javax.resource.spi.LocalTransaction;

//added by VTADev team
import com.ibm.eismocker.rmidriver.RMIObjectClient;
import com.ibm.j2ca.extension.logging.LogUtils;
import java.util.logging.Level;
import javax.resource.spi.CommException;
import com.ibm.j2ca.extension.logging.LogLevel;
import com.ibm.j2ca.extension.logging.LogUtilConstants;

public class VTAManagedConnection extends WBIManagedConnection {

	//added by VTADev team
	private RMIObjectClient rmiSession = null;
	private VTAManagedConnectionFactory factory=null;
	private LogUtils logUtils = null;

	public VTAManagedConnection(WBIManagedConnectionFactory mcf, Subject subject, WBIConnectionRequestInfo cri) throws ResourceException {
		super(mcf, subject, cri);
		// TODO implement connection creation logic
		//added by VTADev team
		factory = (VTAManagedConnectionFactory)mcf;
		logUtils = this.getLogUtils();
	}

    /**
     * Close the physical connection to EIS
     */
	public void destroy() throws ResourceException {
		rmiSession = null;
	}

    /**
     * This method provides EIS specific information
     *
     * @returns	metaData	ManagedConnectionMetaData
     */
	public ManagedConnectionMetaData getMetaData() throws ResourceException {
		return new WBIManagedConnectionMetaData("VTA", "VTA", 1,"VTA");
	}

    /**
     * Extract and use the credentials provided in the PasswordCredential instance passed; 
     * the base classes will provide values from either the Subject for container-managed 
     * sign-on or a WBIConnectionSpec instance for component-managed sign-on as appropriate.
     * <p>
	 * Check if the reauthentication flag is true and reset the connection authentication 
	 * appropriately; this flag should only be set to true if the developer updates the 
	 * deployment descriptor.  If you dont support it, you shouldnt worry about it.
	 * <p>
	 * Return a WBIConnection instance
	 *
	 * @param pc				Password credentials
	 * @param reauthenticate    Boolean to reset the connection
	 * @return 					WBIConnection instance
     */
	public Object getWBIConnection(PasswordCredential pc, boolean reauthenticate) throws ResourceException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAMANAGEDCONN, "getWBIConnection");

		if(rmiSession == null){
			String host = factory.getHostname();
			String port = factory.getPortnumber();
			//log the properties		
			if (logUtils.isTraceEnabled(Level.FINE)) {
				logUtils.trace(Level.FINEST, VTAManagedConnection.class.getName(), "getWBIConnection", "The host name is " + host);
				logUtils.trace(Level.FINEST, VTAManagedConnection.class.getName(), "getWBIConnection", "The port is " + port);
			}
			if(host!=null && host.length()>0 && port != null && port.length()>0){
				try{
					rmiSession = new RMIObjectClient(host, Integer.parseInt(port));
				}catch(Exception e){//fail to establish the connection.
					logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAMANAGEDCONN,"getWBIConnection", "6011");//$NON-NLS-1$
					throw new CommException("The adapter failed to establish the connection with EIS Mocker.",e);
				}
			}else{
				//throw new ResourceException("The property value of host or port is invalid.");				
			}
			//For EMD, don't need to establish the connection.
//			if(rmiSession == null){//fail to establish the connection.
//				logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAAdapterConstants.VTAMANAGEDCONN,"getWBIConnection", "6011");//$NON-NLS-1$
//				throw new CommException("The adapter failed to establish the connection with EIS Mocker.");
//			}
		}
		
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAMANAGEDCONN, "getWBIConnection");

		return new VTAConnection(this);
	}

	/**
     * Does the EIS support local transaction?  Provide a LocalTransaction
     * implementation and return the wrapper.
	 * 
	 * @return  new instance of WBILocalResourceWrapper 
     * @see 	javax.resource.spi.ManagedConnection#getLocalTransaction()
     */
    public LocalTransaction getLocalTransaction() throws ResourceException { 
     	VTALocalTransaction transaction = new VTALocalTransaction(this);
        return new WBILocalTransactionWrapper(transaction, this);
    }

    //added by VTADev team
	public RMIObjectClient getEISConnection(){
		return this.rmiSession;
	}
     
}