/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.outbound;

import java.util.logging.Level;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ResourceAdapter;
import javax.security.auth.Subject;

import com.ibm.j2ca.base.WBIConnectionRequestInfo;
import com.ibm.j2ca.base.WBIManagedConnectionFactory;
import com.ibm.j2ca.base.WBIResourceAdapterMetadata;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.vta.VTAWSAResourceAdapterMetadata;

public class VTAManagedConnectionFactory extends WBIManagedConnectionFactory{
		
	private static final long serialVersionUID = 2;
	//added by VTADev team
	private String hostname = null;
	private String portnumber = null; 

	/**
	 * This method is used by the JCA container in a managed environment; the factory 
	 * generated is handed off to CCI clients enabling them to generate handles to the 
	 * physical EIS connection.
     *
     * @param 	connMgr ConnectionManager
     * @return 	cf		VTAConnectionFactory
	 */
	public Object createConnectionFactory(ConnectionManager connMgr) throws ResourceException {
		return new VTAConnectionFactory(connMgr, this);	
	}


    /**
	 * This method is used by the JCA container to acquire a physical connection 
	 * to the EIS instance. 
	 * 
	 * @param	subject
	 * @param	cri		ConnectionRequestInfo object
	 * @return 	mconn	VTAWBIManagedConnection
	 */
	public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo cri) throws ResourceException {
	        VTAManagedConnection conn = new VTAManagedConnection(this, subject, (WBIConnectionRequestInfo)cri);
	        return conn;
	}

	/**
	 * This method is to get more information specific to a resource adapter implementation 
	 * like name,vendor,version,support local transaction and shortname.
    */
    public WBIResourceAdapterMetadata getResourceAdapterMetadata(){
    	VTAWSAResourceAdapterMetadata vtaRAMetadata =null;
    	try{
    		vtaRAMetadata = new VTAWSAResourceAdapterMetadata();
    	}catch(Exception e){
    	 	getLogUtils().log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, VTAManagedConnectionFactory.class.getName(), "getResourceAdapterMetadata", "6807", new Object[] {});
    	}
    	return vtaRAMetadata;
    }
	
    //added the following methods by VTADev team
	public String getHostname(){
		return this.hostname;
	}
	public void setHostname(String host){
		this.hostname = host;
	}
	public String getPortnumber(){
		return this.portnumber;
	}
	public void setPortnumber(String port){
		this.portnumber = port;
	}
	public void setResourceAdapter(ResourceAdapter resourceAdapter)throws ResourceException {
		super.setResourceAdapter(resourceAdapter);
	}

}