/*
 * IBM Confidential
 * OCO Source Materials
 * (C) Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.outbound.commands;

import com.ibm.despi.InputCursor;
import com.ibm.j2ca.extension.commandpattern.CommandForCursor;
import com.ibm.j2ca.extension.metadata.Type;
import javax.resource.ResourceException;

//added by VTADev team
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.eismocker.rmidriver.DataMap;
import com.ibm.eismocker.rmidriver.IDataObjectStore;
import com.ibm.eismocker.rmidriver.RMIObjectClient;
import java.rmi.RemoteException;
import java.util.logging.Level;
import javax.resource.spi.CommException;
import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.vta.VTAConstants;
import com.ibm.j2ca.vta.util.VTAUtil;

/**
 * If there is some common functionality that all commands have and can be abstracted
 * it is recommended to implement inheritance with this BaseCommand and all specific ones
 * extending the BaseCommand.
 */
public class VTABaseCommand extends CommandForCursor {
	private static final String CLASSNAME = "VTABaseCommand";

	//added by VTADev team
	private RMIObjectClient client = null;
	private IDataObjectStore dataObjStore = null;
	private String operationName =null;
	private LogUtils logUtils =null;
	private String funcName="";
	private String bgVerb="";
	private boolean isBG=false;

	/**
	 * 
	 */
	public VTABaseCommand() {
		super();
		// TODO Auto-generated constructor stub
	}

	//added by VTADev team
	public VTABaseCommand(String nodeLevelOperation){
		operationName = nodeLevelOperation;
		logUtils = getLogUtils();
	}

	 /**
     * execute
     * 
     * @param	inputObject
     * @param	metadata
     * @see com.ibm.j2ca.extension.commandpattern.Command#execute(commonj.sdo.DataObject)
     */
    public void execute(InputCursor inputObject, Type metadata) throws ResourceException {
		logUtils = getLogUtils();
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTABASECOMMAND, "execute");

		try{
			client = (RMIObjectClient)this.getConnection();
			dataObjStore = client.getDataObjectStore();
		}catch(RemoteException re){
			logUtils.trace(Level.FINEST, VTAConstants.VTABASECOMMAND, "execute","The Adapter failed to establish session with EIS Mocker", re);			
			throw new CommException("The Adapter failed to establish session with EIS Mocker",re);			
		}
		try {
			DataMap inputDataMap = null;
			if(this.getIsBG()){
				String boName = metadata.getName();
				String bgName = boName + "BG";
				inputDataMap = new DataMap(bgName);
				DataMap boDataMap = inputDataMap.createDataMap(boName, false);
				inputDataMap.set(VTAConstants.VERB, this.bgVerb);
				VTAUtil.copyRecordToDataMap(inputObject, metadata, boDataMap,logUtils);
			}else{
				inputDataMap = VTAUtil.convertRecordToDataMap(inputObject, metadata,this,logUtils);
			}
			DataMap outputDataMap = dataObjStore.sendDataMap(inputDataMap,this.getFuncName());
			this.setEisRepresentation(outputDataMap);
			if(WBIInteractionSpec.EXISTS_OP.equalsIgnoreCase(operationName)){
				this.setEisRepresentation(new Boolean(true));				
			}
		}catch (RemoteException re) {
			logUtils.trace(Level.FINEST, VTAConstants.VTABASECOMMAND, "execute","The Adapter failed to send data to EIS Mocker", re);			
			throw new CommException("The Adapter failed to send data to EIS Mocker",re);
		}catch (Exception e) {
			throw new ResourceException("The execute call on the VTABaseCommand instance has failed",e);
		}
		
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTABASECOMMAND, "execute");
    }

    protected int maxRecords;
    private String key;
    
	
    /**
     * @return Returns the maxRecords.
     */
    public int getMaxRecords() {
        return maxRecords;
    }

    /**
     * @param maxRecords
     *            The maxRecords to set.
     */
    public void setMaxRecords(int maxRecords) {
        this.maxRecords = maxRecords;
    }

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	//the following methods have been added by VTADev team
	public void setBGVerb(String verb){
		this.bgVerb = verb;
	}
	public String getBGVerb(){
		return this.bgVerb;
	}
	public void setFuncName(String op){
		this.funcName = op;
	}
	public String getFuncName(){
		return this.funcName;
	}
	public void setIsBG(boolean bg){
		this.isBG = bg;
	}
	public boolean getIsBG(){
		return this.isBG;
	}
	//End
}