/*
 * IBM Confidential
 * OCO Source Materials
 * (C) Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.outbound.commands;

import java.util.HashMap;
import javax.resource.ResourceException;
import com.ibm.j2ca.extension.emd.discovery.WBIMetadataDiscoveryImpl;
import com.ibm.j2ca.extension.commandpattern.CommandForCursor;
import com.ibm.j2ca.extension.commandpattern.CommandFactoryForCursor;
import com.ibm.j2ca.extension.commandpattern.NodeLevelOperations;
import com.ibm.j2ca.extension.metadata.Type;

public class VTACommandFactory implements CommandFactoryForCursor {
	private static final String CLASSNAME = "VTACommandFactory";
	
	public VTACommandFactory() {
        super();
    }
    
    public CommandForCursor createCommand(String nodeLevelOperation) throws ResourceException {
		// TODO Auto-generated method stub
		//return createCommand(nodeLevelOperation,null);
    	//added by VTADev team
		return new VTABaseCommand(nodeLevelOperation);
	}

    /**
	 * The isOOType method returns true or false depending on whether the adapter
	 * supports a true CRUD pattern or is a function based adapter. If the adapter 
	 * is not OOType() then the CommandManager would not create child commands the
	 * Command hierarchy would be just one Command at the top level SDO.
	 *
  	 * @return	Return true if the adapter supports a CRUD pattern as an Object Oriented manner
  	 *			and is not function based.
     * @see com.ibm.j2c.commandpattern.CommandFactory#isOOType()
     */
    public boolean isOOType() {
        return false; //value changed by VTADev team 
    }

    HashMap functionBindings;
    int maxRecords;

	/**
	 * The createCommand method should return an instance of CommandForCursor class
	 * that corresponds to the specific function name passed in the interaction spec
	 *
	 * @param	function
	 * @param	metadata	
	 * @return				returns an instance of the corresponding command class
	 */
    public CommandForCursor createCommand(String functionName, Type metadata) throws ResourceException {
		WBIMetadataDiscoveryImpl.getLogUtils().traceMethodEntrance(CLASSNAME, "execute()");
		
		VTABaseCommand command = null;

		try {

			if (functionName.equals(NodeLevelOperations.CREATE_NODE)) {
					command = new VTACreateCommand();
			} else if (functionName.equals(NodeLevelOperations.DELETE_NODE)) {
					command = new VTADeleteCommand();
			} else if (functionName.equals(NodeLevelOperations.UPDATE_NODE)) {
					command = new VTAUpdateCommand();
			} else if (functionName.equals(NodeLevelOperations.RETRIEVE_STRUCTURE)) {
					command = new VTARetrieveCommand();
			} else if (functionName.equals(NodeLevelOperations.RETRIEVE_ALL)) {
					command = new VTARetrieveAllCommand();
			} else if (functionName.equals(NodeLevelOperations.NO_OPERATION)) {
					command = new VTANoOperationCommand();
			} else {
					command = new VTABaseCommand();
			}

			command.setMaxRecords(maxRecords);
			command.setMetadata(metadata);
		}catch (Exception e) {
	        throw new ResourceException(e);
	    }
	    WBIMetadataDiscoveryImpl.getLogUtils().traceMethodExit(CLASSNAME, "execute()");
        return command;
    }

    /**
     * @return Returns the maxRecords.
     */
    public int getMaxRecords() {
        return maxRecords;
    }

    /**
     * @param maxRecords
     *            The maxRecords to set.
     */
    public void setMaxRecords(int maxRecords) {
        this.maxRecords = maxRecords;
    }
}